#include <Adafruit_ST77xx.h>
#include <Adafruit_ST7789.h>
#include <Adafruit_ST7735.h>

#include <Adafruit_GrayOLED.h>
#include <gfxfont.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SPITFT.h>
#include <Adafruit_SPITFT_Macros.h>

#define BUZZER_PIN     7
#define TFT_CS        10
#define TFT_RST        8 
#define TFT_DC         9
#define PADDING       16
#define MAX_DISTANCE 100
#define MAX_INTERVAL 15625
#define MIN_INTERVAL 1562
#define MIN_BEEP_DIST 10
#define MAX_BEEP_DIST 50
#define ECHO_PIN 2 // attach pin D2 Arduino to pin Echo of HC-SR04
#define TRIG_PIN 3 //attach pin D3 Arduino to pin Trig of HC-SR04
#define ST77XX_LIME    0x07FF
#define ST77XX_DARK_GREEN 0x3AA3

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);

long duration; // variable for the duration of sound wave travel
static volatile int distance; // variable for the distance measurement

ISR(TIMER1_COMPA_vect) {
  if (distance <= 50) {
      digitalWrite(BUZZER_PIN, 1);
  }
}

ISR(TIMER1_COMPB_vect) {
  digitalWrite(BUZZER_PIN, 0);
}

void setup() {
  pinMode(BUZZER_PIN, OUTPUT); 
  tft.initR(INITR_144GREENTAB);
  pinMode(TRIG_PIN, OUTPUT); 
  pinMode(ECHO_PIN, INPUT);
  delay(1000);

  cli();

  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;
  
  OCR1A = 15625;     
  OCR1B = 1562;
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12) | (1 << CS10);    // 1024 prescaler 
  TIMSK1 |= (1 << OCIE1A) | (1 << OCIE1B);  // enable timer compare interrupt
  sei();
}

void loop() {
  digitalWrite(TRIG_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);
  duration = pulseIn(ECHO_PIN, HIGH);
  distance = duration * 0.034 / 2; 
  printUI(distance);
  OCR1A = beep_freq(distance);
  delay(1000);
}

int beep_freq(uint8_t distance) {
  float a = (MAX_INTERVAL - MIN_INTERVAL) / (MAX_BEEP_DIST - MIN_BEEP_DIST);
  float b = (MIN_INTERVAL - MIN_BEEP_DIST * a);
  return a * distance - b;
}
void printUI(uint8_t distance) {
  tft.fillScreen(ST77XX_BLACK);
  printDistance(distance, ST77XX_WHITE);
  printDistanceCircle(distance);
  printSelf();
}

void printDistance(uint8_t distance, uint16_t color) {
  tft.setCursor(0, 0);
  tft.setTextColor(color);
  tft.setTextWrap(true);
  tft.setTextSize(3);
  tft.print(distance);
  tft.print("cm");
}

void printSelf() {
    tft.fillCircle(tft.width() / 2, tft.height() / 2, 4, ST77XX_WHITE);
}

void printDistanceCircle(uint8_t distance) {
  uint16_t color;
  if (distance >= 80) {
    color = ST77XX_DARK_GREEN;
  } else if (distance >= 60 && distance < 80) {
    color = ST77XX_GREEN;
  } else if (distance >= 40 && distance < 60) {
    color = ST77XX_YELLOW;
  } else if (distance >= 20 && distance < 40) {
    color = ST77XX_ORANGE;
  } else {
    color = ST77XX_RED;
  }
  float radius = 1. * distance / MAX_DISTANCE * (tft.width() / 2 - PADDING / 2);
  tft.drawCircle(tft.width() / 2, tft.height() / 2, radius, color);
}
